# Generated by Django 4.1.7 on 2023-03-16 20:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('scheduler', '0009_alter_jobarg_arg_type_alter_jobarg_val_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='Queue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': [['view', 'Access admin page']],
                'managed': False,
                'default_permissions': (),
            },
        ),
    ]
