# Generated by Django 4.2.5 on 2023-10-08 16:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('scheduler', '0015_rename_cronjob_crontask_and_more'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='JobArg',
            new_name='TaskArg',
        ),
        migrations.RenameModel(
            old_name='JobKwarg',
            new_name='TaskKwarg',
        ),
        migrations.AlterModelOptions(
            name='crontask',
            options={'ordering': ('name',), 'verbose_name': 'Cron Task', 'verbose_name_plural': 'Cron Tasks'},
        ),
        migrations.AlterModelOptions(
            name='repeatabletask',
            options={'ordering': ('name',), 'verbose_name': 'Repeatable Task', 'verbose_name_plural': 'Repeatable Tasks'},
        ),
        migrations.AlterModelOptions(
            name='scheduledtask',
            options={'ordering': ('name',), 'verbose_name': 'Scheduled Task', 'verbose_name_plural': 'Scheduled Tasks'},
        ),
    ]
