# Generated by Django 3.2.12 on 2022-03-29 21:07

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('scheduler', '0002_alter_cronjob_id_alter_repeatablejob_id_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cronjob',
            name='cron_string',
            field=models.CharField(help_text='Define the schedule in a crontab like syntax. Times are in UTC.',
                                   max_length=64, verbose_name='cron string'),
        ),
        migrations.AlterField(
            model_name='cronjob',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='repeatablejob',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='repeatablejob',
            name='interval_unit',
            field=models.CharField(
                choices=[('seconds', 'seconds'), ('minutes', 'minutes'), ('hours', 'hours'), ('days', 'days'),
                         ('weeks', 'weeks')], default='hours', max_length=12, verbose_name='interval unit'),
        ),
        migrations.AlterField(
            model_name='scheduledjob',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.CreateModel(
            name='JobKwarg',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('arg_type', models.CharField(
                    choices=[('str_val', 'string'), ('int_val', 'int'), ('bool_val', 'boolean'),
                             ('datetime_val', 'Datetime')], default='str_val', max_length=12,
                    verbose_name='Argument Type')),
                ('str_val', models.CharField(blank=True, max_length=255, verbose_name='String Value')),
                ('int_val', models.IntegerField(blank=True, null=True, verbose_name='Int Value')),
                ('bool_val', models.BooleanField(default=False, verbose_name='Boolean Value')),
                ('datetime_val', models.DateTimeField(blank=True, null=True, verbose_name='Datetime Value')),
                ('object_id', models.PositiveIntegerField()),
                ('key', models.CharField(max_length=255)),
                ('content_type',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'ordering': ['id'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='JobArg',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('arg_type', models.CharField(
                    choices=[('str_val', 'string'), ('int_val', 'int'), ('bool_val', 'boolean'),
                             ('datetime_val', 'Datetime')], default='str_val', max_length=12,
                    verbose_name='Argument Type')),
                ('str_val', models.CharField(blank=True, max_length=255, verbose_name='String Value')),
                ('int_val', models.IntegerField(blank=True, null=True, verbose_name='Int Value')),
                ('bool_val', models.BooleanField(default=False, verbose_name='Boolean Value')),
                ('datetime_val', models.DateTimeField(blank=True, null=True, verbose_name='Datetime Value')),
                ('object_id', models.PositiveIntegerField()),
                ('content_type',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'ordering': ['id'],
                'abstract': False,
            },
        ),
    ]
