# Generated by Django 4.2.5 on 2023-10-08 16:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('scheduler', '0014_alter_cronjob_created_alter_cronjob_modified_and_more'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='CronJob',
            new_name='CronTask',
        ),
        migrations.RenameModel(
            old_name='RepeatableJob',
            new_name='RepeatableTask',
        ),
        migrations.RenameModel(
            old_name='ScheduledJob',
            new_name='ScheduledTask',
        ),
    ]
